using InternetShopContracts.DataSearchModels;
using InternetShopContracts.DataViewModels;
using InternetShopContracts.LogicsContracts;
using InternetShopForms.Orders;
using InternetShopForms.Products;
using WinFormsLibrary1.HelperClasses;

namespace InternetShopForms
{
    public partial class FormOrdersList : Form
    {
        private IOrderLogic _orderLogic;
        private IProductLogic _productLogic;
        public FormOrdersList(IOrderLogic orderLogic, IProductLogic productLogic)
        {
            InitializeComponent();
            _orderLogic = orderLogic;
            _productLogic = productLogic;

            List<(string, string, float)> configureColumns = [
                ("ID", "Id", 1.0f),
                (" ", "CustomerFIO", 1.0f),
                ("", "ProductsString", 1.0f),
                ("Email ", "CustomerEmail", 1.0f),
            ];

            tableComponent.ConfigureColumns(configureColumns);
            tableComponent.dataGridView1.AllowUserToDeleteRows = false;

            tableComponent.dataGridView1.Columns[0].Visible = false;
            KeyPreview = true;
        }

        private void productsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormProductsList));
            if (service is FormProductsList form)
            {
                form.ShowDialog();
            }
        }

        private void FormOrdersList_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var orders = _orderLogic.ReadList();
                tableComponent.FillData(orders);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormOrdersList_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control)
            {
                switch (e.KeyCode)
                {
                    case Keys.A:
                        CreateOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.U:
                        EditOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.D:
                        DeleteOrder();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.S:
                        CreateWord();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.C:
                        CreateExcel();
                        e.SuppressKeyPress = true;
                        break;
                    case Keys.T:
                        CreatePDF();
                        e.SuppressKeyPress = true;
                        break;
                }
            }
        }

        private void CreateOrder()
        {
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }

            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void EditOrder()
        {
            OrderViewModel? selectedOrder = null;
            try
            {
                selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();
            }
            catch (Exception ex)
            {
                MessageBox.Show(" ");
                return;
            }
            var service = Program.ServiceProvider?.GetService(typeof(FormOrderEdit));
            if (!(service is FormOrderEdit form))
            {
                return;
            }
            form.OrderId = Convert.ToInt32(selectedOrder.Id);
            if (form.ShowDialog() == DialogResult.OK)
            {
                LoadData();
            }
        }

        private void DeleteOrder()
        {
            try
            {
                OrderViewModel? selectedOrder = null;
                try
                {
                    selectedOrder = tableComponent.GetSelectedObject<OrderViewModel>();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(" ");
                    return;
                }

                var result = MessageBox.Show(
                    "   ?",
                    "",
                    MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question
                );

                if (result == DialogResult.Yes)
                {
                    if (_orderLogic.Delete(new OrderSearchModel { Id = selectedOrder.Id }))
                    {
                        MessageBox.Show(" ");
                    }
                    else
                    {
                        throw new Exception(" ");
                    }
                    LoadData();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private enum ReportType
        {
            Word = 0,
            Excel = 1,
            PDF = 2
        }

        private string GetFileFilter(ReportType reportType)
        {
            if (reportType == ReportType.Excel)
            {
                return "Excel files (*.xlsx)|*.xlsx";
            }
            else if (reportType == ReportType.PDF)
            {
                return "PDF files (*.pdf)|*.pdf";
            }
            else
            {
                return "Word files (*.docx)|*.docx";
            }
        }

        private string? GetExportFileName(ReportType reportType)
        {
            using (SaveFileDialog openFileDialog = new SaveFileDialog())
            {
                openFileDialog.InitialDirectory = "d:\\tmp";
                openFileDialog.Filter = GetFileFilter(reportType) + "|All files (*.*)|*.*";
                openFileDialog.FilterIndex = 1;
                openFileDialog.RestoreDirectory = true;

                if (openFileDialog.ShowDialog() == DialogResult.OK)
                {
                    return openFileDialog.FileName;
                }
            }
            return null;
        }

        private void CreateWord()
        {
            string? exportFileName = GetExportFileName(ReportType.Word);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            var orders = _orderLogic.ReadList();
            try
            {
                string[] headerRow1 = {
                    "ID",
                    " ",
                    " ",
                    "",
                };

                string[] headerRow2 = {
                    "ID",
                    "",
                    "Email",
                    "",
                };

                List<float> columnWidths = [3.0f, 3.0f, 3.0f, 3.0f];

                var mergeColumns = new List<(int StartColumn, int EndColumn)> { (1, 2) };

                var columnPropertyMapping = new Dictionary<int, string>
                    {
                        { 0, "Id" },
                        { 1, "CustomerFIO" },
                        { 2, "CustomerEmail" },
                        { 3, "ProductsString" },
                    };

                tableWordNoVisibleComponent.CreateTableInWordDocument(
                    exportFileName,
                    "  ",
                    mergeColumns,
                    columnWidths,
                    headerRow1,
                    headerRow2,
                    columnPropertyMapping,
                    orders
                );
                MessageBox.Show("  ", " ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, " ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CreateExcel()
        {
            string? exportFileName = GetExportFileName(ReportType.Excel);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            var orders = _orderLogic.ReadList();
            try
            {
                componentExcelWithImage.CreateExcelWithImages(exportFileName, "", orders.Select(x => x.ImagePath).ToArray());
                MessageBox.Show("  ", " ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, " ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        private void CreatePDF()
        {
            string? exportFileName = GetExportFileName(ReportType.PDF);
            if (string.IsNullOrEmpty(exportFileName))
            {
                return;
            }
            var orders = _orderLogic.ReadList();
            var products = _productLogic.ReadList();
            Dictionary<string, double> productsStat = new Dictionary<string, double>();
            foreach (var order in orders)
            {
                foreach (var product in order.ProductNames)
                {
                    if (productsStat.ContainsKey(product))
                    {
                        productsStat[product]++;
                    }
                    else
                    {
                        productsStat.Add(product, 1);
                    }
                }
            }

            foreach (var product in products)
            {
                if (!productsStat.ContainsKey(product.Name))
                {
                    productsStat.Add(product.Name, 0);
                }
            }

            List<ChartData> charts = new List<ChartData>();
            
            foreach (var product in productsStat.Keys)
            {
                ChartData chartData = new ChartData();
                Dictionary<string, double> data = new Dictionary<string, double>();
                data.Add(product, productsStat[product]);
                chartData.SeriesName = product;
                chartData.Data = data;
                charts.Add(chartData);
            }

            try
            {
                pdfHistogram.CreateHistogramPdf(
                    exportFileName,
                    "  ",
                    " ",
                    OxyPlot.Legends.LegendPosition.RightTop,
                    charts
                );
                MessageBox.Show("  ", " ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("    :\n" + ex.Message, " ", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void reportOrderImagesExcelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateExcel();
        }

        private void reportOrdersWordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateWord();
        }

        private void reportProductsPDFToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreatePDF();
        }
    }
}
